#!/usr/bin/env bash

# Usage: ./install.sh USER PGDATA


if [ $# != 2 ]
then
  echo "Error: 参数不足"
  echo "用法: ./install.sh USER PGDATA"
  echo "- USER: 数据库安装用户（一般为 omm 或者 postgres）"
  echo "- PGDATA: 数据库数据目录（可用于 gs_ctl 等指令的-D参数）"
  exit 1
fi

function log() {
  message="[$(date +%Y-%m-%d\ %H:%M:%S)]: $1"
  echo $message
  if [ $# -eq 2 ];then
   echo $message >> $2
  fi
}

VERSION=$(cat version.txt)
if [ ! ${VERSION} ]; then
    log "找不到版本号文件 version.txt"
    exit 1
fi

CUR_DIR=$(dirname $(readlink -f $0))
log "MogHA 安装目录为：${CUR_DIR}"

USER=$1
GROUP_ID=$(grep -E "^${USER}" /etc/passwd | awk -F":" '{print $3}')
GROUP=$(grep -E ":${GROUP_ID}" /etc/group | awk -F":" '{print $1}')
log "数据库安装用户为：${USER}, 所在用户组为：${GROUP}"

arg2=$2
PGDATA=${arg2%*/}
log "数据库数据目录为：${PGDATA}"
ldenv=$(sudo -Hiu ${USER} env | grep 'LD_LIBRARY_PATH')
LD_LIBRARY_PATH=${ldenv#*=}
log "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}"
ghenv=$(sudo -Hiu ${USER} env | grep 'GAUSSHOME')
GAUSSHOME=${ghenv#*=}
log "GAUSSHOME=${GAUSSHOME}"
hostenv=$(sudo -Hiu ${USER} env | grep 'PGHOST')
PGHOST=${hostenv#*=}
log "PGHOST=${PGHOST}"
if [ "${PGHOST}" == "" ]
then
  PGHOST_ENV=""
else
  PGHOST_ENV="PGHOST=${PGHOST}"
fi
DB_PORT=$(grep -E "^port\s*=" ${PGDATA}/postgresql.conf |  tr -cd "[0-9]")
if [ ! ${DB_PORT} ]; then
    DB_PORT=5432
fi
log "数据库端口: ${DB_PORT}"

SYSSVC_DIR="/usr/lib/systemd/system/"

ARCH=$(uname -m)
log "当前系统架构为：${ARCH}"
BINARY=mogha-${VERSION}-${ARCH}
MOGHA_EXE=${BINARY}/${BINARY}

if [ ! -f "${MOGHA_EXE}" ]
then
  log "不支持当前系统架构"
  exit 1
fi


function replaceMacro() {
  local filename="$1"
  local macro="$2"
  local value="$3"

  sed -i "s#${macro}#${value}#g" ${filename}
}

log "修改安装目录及其子目录或文件的用户和用户组..."
chown -R ${USER}:${GROUP} ${CUR_DIR}

log "链接系统的 python3 解释器到虚拟环境..."
grep -rl "__INSTALLDIR__/prod_venv" ${CUR_DIR}/prod_venv/bin >/dev/null 2>&1
if [ $? == 0 ]
then
  sed -i "s#__INSTALLDIR__#${CUR_DIR}#g" `grep -rl "__INSTALLDIR__/prod_venv" ${CUR_DIR}/prod_venv/bin`
  PYTHON3_PATH=$(command -v python3)
  if [ $? != 0 ]
  then
    log "在当前系统中，未找到 python3 命令, 退出安装"
    exit 1
  else
    log "当前系统中 Python3 的路径为：${PYTHON3_PATH}"
  fi
  PYTHON3_VERSION=$(python3 -V 2>&1 | sed 's/.* \([0-9]\).\([0-9]\).*/\1.\2/')
  log "Python3 的版本为：${PYTHON3_VERSION}"
  rm -f ${CUR_DIR}/prod_venv/bin/python ${CUR_DIR}/prod_venv/bin/python3
  ln -s ${PYTHON3_PATH} ${CUR_DIR}/prod_venv/bin/python
  ln -s ${PYTHON3_PATH} ${CUR_DIR}/prod_venv/bin/python3
  if [ ${PYTHON3_VERSION} != "3.6" ]
  then
    mv ${CUR_DIR}/prod_venv/lib/python3.6 ${CUR_DIR}/prod_venv/lib/python${PYTHON3_VERSION}
  fi
  log "虚拟环境中 python 解释器链接成功"
fi

SUPERVISORD=${CUR_DIR}/prod_venv/bin/supervisord


log "生成 mogha.service 文件..."
rm -f ${CUR_DIR}/mogha.service
cp ${CUR_DIR}/mogha.service.tmpl ${CUR_DIR}/mogha.service
chown ${USER}:${GROUP} ${CUR_DIR}/mogha.service
chmod 755 ${CUR_DIR}/mogha.service
replaceMacro ${CUR_DIR}/mogha.service "__SUPERVISORD__" ${SUPERVISORD}
replaceMacro ${CUR_DIR}/mogha.service "__GAUSSHOME__" ${GAUSSHOME}
replaceMacro ${CUR_DIR}/mogha.service "__PGDATA__" ${PGDATA}
replaceMacro ${CUR_DIR}/mogha.service "__USER__" ${USER}
replaceMacro ${CUR_DIR}/mogha.service "__LD_LIBRARY_PATH__" ${LD_LIBRARY_PATH}
replaceMacro ${CUR_DIR}/mogha.service "__INSTALLDIR__" ${CUR_DIR}
log "复制 mogha.service 文件到 ${SYSSVC_DIR}"
cp ${CUR_DIR}/mogha.service ${SYSSVC_DIR}
log "重载配置文件"
systemctl daemon-reload
log "mogha 服务注册成功"


SUDOFILE="/etc/sudoers"
chmod +w ${SUDOFILE}
CMD_SYSTEMCTL=$(command -v systemctl)
CMD_IFCONFIG=$(command -v ifconfig)
grep -e "^${USER}\s*ALL" ${SUDOFILE} >/dev/null 2>&1
if [ $? != 0 ]
then
  log "配置 ${USER} 用户的 sudo 权限到 /etc/sudoers 文件"
  echo "${USER}     ALL=(ALL)       NOPASSWD:${CMD_SYSTEMCTL},NOPASSWD:${CMD_IFCONFIG}" >> ${SUDOFILE}
else
  log "用户 ${USER} 已经存在于 /etc/sudoers 文件中，请自行确认权限!"
fi
chmod -w /etc/sudoers


log "生成 supervisord.conf 配置文件..."
rm -f ${CUR_DIR}/supervisord.conf
cp ${CUR_DIR}/supervisord.conf.tmpl ${CUR_DIR}/supervisord.conf
chown ${USER}:${GROUP} ${CUR_DIR}/supervisord.conf
chmod 755 ${CUR_DIR}/supervisord.conf
replaceMacro ${CUR_DIR}/supervisord.conf "__INSTALLDIR__" ${CUR_DIR}
replaceMacro ${CUR_DIR}/supervisord.conf "__USER__" ${USER}
replaceMacro ${CUR_DIR}/supervisord.conf "__MOGHA_BIN__" ${MOGHA_EXE}
replaceMacro ${CUR_DIR}/supervisord.conf "__GAUSSHOME__" ${GAUSSHOME}
replaceMacro ${CUR_DIR}/supervisord.conf "__PGDATA__" ${PGDATA}
replaceMacro ${CUR_DIR}/supervisord.conf "__PGHOST_ENV__" ${PGHOST_ENV}
replaceMacro ${CUR_DIR}/supervisord.conf "__LD_LIBRARY_PATH__" ${LD_LIBRARY_PATH}
log "supervisord.conf 生成成功"


if [ ! -f "${CUR_DIR}/node.conf" ]
then
  log "没有在安装目录中发现 node.conf 配置文件"
  log "生成 node.conf..."
  cp ${CUR_DIR}/node.conf.tmpl ${CUR_DIR}/node.conf
  chown ${USER}:${GROUP} ${CUR_DIR}/node.conf
  chmod 755 ${CUR_DIR}/node.conf
  replaceMacro ${CUR_DIR}/node.conf "__INSTALLDIR__" ${CUR_DIR}
  replaceMacro ${CUR_DIR}/node.conf "__DB_PORT__" ${DB_PORT}
  replaceMacro ${CUR_DIR}/node.conf "__USER__" ${USER}
  replaceMacro ${CUR_DIR}/node.conf "__PGDATA__" ${PGDATA}
  echo "node.conf 生成成功，位于目录：${CUR_DIR}/node.conf"
else
  log "发现 node.conf 已经存在，不再自动生成"
fi

log "MogHA 安装成功!"
echo ""
echo "请修改 ${CUR_DIR}/node.conf 配置文件后再启动服务 !!!"
echo ""
echo "启动 MogHA 的命令:"
echo "    systemctl start mogha"
echo ""
